@@ -42,9 +42,7 @@ complexity.
 It requires python2.6 or above (early versions are not verified).
 """
 
-BUG_REPORTING = "please report bug to terry.yinzhe@gmail.com or https://github.com/terryyin/lizard.\n"
-
-VERSION = "1.7.7"
+VERSION = "1.7.8"
 
 
 import itertools, traceback
@@ -54,6 +52,20 @@ import re
 DEFAULT_CCN_THRESHOLD = 15
 
 
+def analyze(paths, options):
+    ''' This is the most important function of lizard.
+        It analyzes the given paths with the options.
+        Can be used directly by other Python application.
+    '''
+
+    languages = LanguageChooser(
+            includeHashIfConditions = not options.no_preprocessor_count,
+            switchCasesAsOneCondition = options.switchCasesAsOneCondition,)
+
+    files = FilesFilter(options.exclude, options.duplicates).getFileNames(paths)
+    fileAnalyzer = FileAnalyzer(languages, options.extensions)
+    return mapFilesToAnalyzer(files, fileAnalyzer, options.working_threads)
+
 def createCommandLineParser():
     from optparse import OptionParser
     parser = OptionParser(version=VERSION)
@@ -186,9 +198,9 @@ class FileInformation(object):
                 / len(self.function_list) if self.function_list else 0
 
 
-class SourceCodeInforamtionBuilder(object):
+class CodeInfoContext(object):
     """
-        SourceCodeInforamtionBuilder provides the builder to build FileInformation
+        provides the builder to build FileInformation
         The building blocks are:
         START_NEW_FUNCTION
             ADD_TO_FUNCTION_NAME
@@ -205,26 +217,16 @@ class SourceCodeInforamtionBuilder(object):
         self.START_NEW_FUNCTION('')
         self.forgive = False
 
-    def parse_functions(self, tokens, parser):
-        parser.sourceCodeInfoBuilder = self
-        self.current_line = 1 
-        for token in tokens:
-            self.current_line += 1 if token == '\n' else (len(token.splitlines()) - 1)
-            try:
-                if token.isspace():
-                    self.NEW_LINE()
-                elif token.startswith("/*") or token.startswith("//"):
-                    if len(token.splitlines()) > 1:
-                        self.NEW_LINE()
-                    if token[2:].strip().startswith("#lizard forgive"):
-                        self.forgive = True
-                else:
-                    self.TOKEN()
-                    self.fileinfo.token_count+=1
-                    parser.process_token(token)
-            except:
-                raise ParsingError(self.current_line)
-        return self.fileinfo
+    def _new_line(self):
+        self.fileinfo.nloc += 1
+        self.newline = True
+
+    def _count_token(self):
+        self.fileinfo.token_count+=1
+        if self.newline:
+            self.current_function.nloc += 1
+            self.newline = False
+        self.current_function.token_count += 1
 
     def START_NEW_FUNCTION(self, name):
         self.newline = True
@@ -233,16 +235,6 @@ class SourceCodeInforamtionBuilder(object):
     def CONDITION(self):
         self.current_function.cyclomatic_complexity += 1
 
-    def TOKEN(self):
-        if self.newline:
-            self.current_function.nloc += 1
-            self.newline = False
-        self.current_function.token_count += 1
-
-    def NEW_LINE(self):
-        self.fileinfo.nloc += 1
-        self.newline = True
-
     def ADD_TO_LONG_FUNCTION_NAME(self, app):
         self.current_function.add_to_long_name(app)
 
@@ -260,44 +252,34 @@ class SourceCodeInforamtionBuilder(object):
         self.START_NEW_FUNCTION('')
 
 
-class ParsingError(Exception):
+class LineCounter():
 
-    def __init__(self, line_number):
-        self.line_number = line_number
-        self.filename = ''
-        self.code = ''
-
-    def __str__(self):
-        try:
-            line = self.code.splitlines()[self.line_number - 1]
-        except:
-            line = ''
-        return '''!!!Exception Happens!!!
-At %s:%d: '%s'
-If possible, ''' % (self.filename, self.line_number, line) + BUG_REPORTING
+    def extend_tokens(self, tokens, context):
+        self.context = context
+        self.context.current_line = 1 
+        for token in tokens:
+            self.context.current_line += 1 if token == '\n' else (len(token.splitlines()) - 1)
+            if token == "\n":
+                self.context._new_line()
+            elif token.startswith("/*") or token.startswith("//"):
+                if len(token.splitlines()) > 1:
+                    self.context._new_line()
+                if token[2:].strip().startswith("#lizard forgive"):
+                    self.context.forgive = True
+            else:
+                self.context._count_token()
+                yield token
 
 
 class LanguageReaderBase(object):
-    '''
-    This is the base class for any programming language reader.
-    It reader the tokens of a certain language and generate universal_code
-    by calling it's -generate_universal_code- method.
 
-    The derived class should implement a series of 'state' functions. The
-    most basic and start state is _GLOBAL. Derived class must implement _GLOBAL
-    and other states. When the state changes, simply assign the state method to
-    self._state.
-    During each state, call the methods of self.universal_code to build the
-    universal code.
-    '''
-
-    def __init__(self):
+    def process_tokens(self, tokens, context):
         self._state = self._GLOBAL
+        self.context = context
+        for token in tokens:
+            self._state(token)
 
-    def process_token(self, token):
-        return self._state(token)
-
-    def extend_tokens(self, tokens):
+    def extend_tokens(self, tokens, context):
         return tokens
 
 
@@ -331,7 +313,7 @@ class CLikeReader(LanguageReaderBase):
 
     macro_pattern = re.compile(r"#\s*(\w+)\s*(.*)", re.M | re.S)
 
-    def extend_tokens(self, tokens):
+    def extend_tokens(self, tokens, context):
         # handle c preprocessors
         for token in tokens:
             m = self.macro_pattern.match(token)
@@ -346,31 +328,26 @@ class CLikeReader(LanguageReaderBase):
     def _is_condition(self, token):
         return token in self.conditions
 
-    def process_token(self, token):
-        if token.startswith("#") and self._state != self._IMP:
-            return
-        return super(CLikeReader, self).process_token(token)
-
     def _GLOBAL(self, token):
         if token == '(':
             self.bracket_level += 1
             self._state = self._DEC
-            self.sourceCodeInfoBuilder.ADD_TO_LONG_FUNCTION_NAME(token)
+            self.context.ADD_TO_LONG_FUNCTION_NAME(token)
         elif token == '::':
             self._state = self._NAMESPACE
         else:
-            self.sourceCodeInfoBuilder.START_NEW_FUNCTION(token)
+            self.context.START_NEW_FUNCTION(token)
             if token == 'operator':
                 self._state = self._OPERATOR
 
     def _OPERATOR(self, token):
         if token != '(':
             self._state = self._GLOBAL
-        self.sourceCodeInfoBuilder.ADD_TO_FUNCTION_NAME(' ' + token)
+        self.context.ADD_TO_FUNCTION_NAME(' ' + token)
 
     def _NAMESPACE(self, token):
         self._state = self._OPERATOR if token == 'operator'  else self._GLOBAL
-        self.sourceCodeInfoBuilder.ADD_TO_FUNCTION_NAME("::" + token)
+        self.context.ADD_TO_FUNCTION_NAME("::" + token)
 
     def _DEC(self, token):
         if token in ('(', "<"):
@@ -380,13 +357,13 @@ class CLikeReader(LanguageReaderBase):
             if (self.bracket_level == 0):
                 self._state = self._DEC_TO_IMP
         elif self.bracket_level == 1:
-            self.sourceCodeInfoBuilder.PARAMETER(token)
+            self.context.PARAMETER(token)
             return
-        self.sourceCodeInfoBuilder.ADD_TO_LONG_FUNCTION_NAME(" " + token)
+        self.context.ADD_TO_LONG_FUNCTION_NAME(" " + token)
 
     def _DEC_TO_IMP(self, token):
         if token == 'const':
-            self.sourceCodeInfoBuilder.ADD_TO_LONG_FUNCTION_NAME(" " + token)
+            self.context.ADD_TO_LONG_FUNCTION_NAME(" " + token)
         elif token == '{':
             self.br_count += 1
             self._state = self._IMP
@@ -411,10 +388,10 @@ class CLikeReader(LanguageReaderBase):
                 self.br_count -= 1
                 if self.br_count == 0:
                     self._state = self._GLOBAL
-                    self.sourceCodeInfoBuilder.END_OF_FUNCTION()
+                    self.context.END_OF_FUNCTION()
                     return
         if self._is_condition(token):
-            self.sourceCodeInfoBuilder.CONDITION()
+            self.context.CONDITION()
 
 
 class ObjCReader(CLikeReader):
@@ -428,12 +405,12 @@ class ObjCReader(CLikeReader):
             super(ObjCReader, self)._DEC_TO_IMP(token)
             if self._state == self._GLOBAL:
                 self._state = self._OBJC_DEC_BEGIN
-                self.sourceCodeInfoBuilder.START_NEW_FUNCTION(token)
+                self.context.START_NEW_FUNCTION(token)
 
     def _OBJC_DEC_BEGIN(self, token):
         if token == ':':
             self._state = self._OBJC_DEC
-            self.sourceCodeInfoBuilder.ADD_TO_FUNCTION_NAME(token)
+            self.context.ADD_TO_FUNCTION_NAME(token)
         elif token == '{':
             self.br_count += 1
             self._state = self._IMP
@@ -443,7 +420,7 @@ class ObjCReader(CLikeReader):
     def _OBJC_DEC(self, token):
         if token == '(':
             self._state = self._OBJC_PARAM_TYPE
-            self.sourceCodeInfoBuilder.ADD_TO_LONG_FUNCTION_NAME(token)
+            self.context.ADD_TO_LONG_FUNCTION_NAME(token)
         elif token == ',':
             pass
         elif token == '{':
@@ -451,12 +428,12 @@ class ObjCReader(CLikeReader):
             self._state = self._IMP
         else:
             self._state = self._OBJC_DEC_BEGIN
-            self.sourceCodeInfoBuilder.ADD_TO_FUNCTION_NAME(" " + token)
+            self.context.ADD_TO_FUNCTION_NAME(" " + token)
 
     def _OBJC_PARAM_TYPE(self, token):
         if token == ')':
             self._state = self._OBJC_PARAM
-        self.sourceCodeInfoBuilder.ADD_TO_LONG_FUNCTION_NAME(" " + token)
+        self.context.ADD_TO_LONG_FUNCTION_NAME(" " + token)
 
     def _OBJC_PARAM(self, token):
         self._state = self._OBJC_DEC
@@ -499,41 +476,26 @@ default_language_chooser = LanguageChooser(
 )
 
 class FileAnalyzer:
-    ''' A FileAnalyzer works as a function. It takes filename as parameter.
-        Returns a list of function infos in this file.
-    '''
 
     def __init__(self, languageChooser = default_language_chooser, extensions = []):
         self.extensions = extensions
         self.languageChooser = languageChooser
 
     def __call__(self, filename):
-        try:
-            with open(filename) as f:
-                return self.analyze_source_code(filename, f.read())
-        except Exception as e:
-            msg = '\n'.join(traceback.format_exception_only(type(e), e))
-            msg+= "\nIf you think this is a bug, "+ BUG_REPORTING
-            sys.stderr.write(msg)
+        with open(filename) as f:
+            return self.analyze_source_code(filename, f.read())
 
     def analyze_source_code(self, filename, code):
-        try:
-            reader = self.languageChooser.get_reader_by_file_name_otherwise_default(filename)
-            return self.analyze_source_code_with_parser(filename, code, reader)
-        except ParsingError as e:
-            e.filename = filename
-            e.code = code
-            raise
+        reader = self.languageChooser.get_reader_by_file_name_otherwise_default(filename)
+        return self.analyze_source_code_with_parser(filename, code, reader)
 
     def analyze_source_code_with_parser(self, filename, code, parser):
         tokens = generate_tokens(code)
-        tokens = parser.extend_tokens(tokens)
-        for extension in self.extensions:
-            tokens = extension.extend_tokens(tokens)
-        result = SourceCodeInforamtionBuilder(filename).parse_functions(tokens, parser)
-        for extension in self.extensions:
-            result.wordCount = extension.result
-        return result
+        context = CodeInfoContext(filename)
+        for extension in [parser, LineCounter()] + self.extensions:
+            tokens = extension.extend_tokens(tokens, context)
+        parser.process_tokens(tokens, context)
+        return context.fileinfo
 
 analyze_file = FileAnalyzer()
 
@@ -920,31 +882,16 @@ class FilesFilter(object):
             self.hash_set.add(fhash)
             return True
 
-def analyze(paths, options):
-    ''' This is the most important function of lizard.
-        It analyze the given paths with the options.
-        Can be used directly by other Python application.
-    '''
-
-    languages = LanguageChooser(
-            includeHashIfConditions = not options.no_preprocessor_count,
-            switchCasesAsOneCondition = options.switchCasesAsOneCondition,)
-
-    files = FilesFilter(options.exclude, options.duplicates).getFileNames(paths)
-    fileAnalyzer = FileAnalyzer(languages, options.extensions)
-    return mapFilesToAnalyzer(files, fileAnalyzer, options.working_threads)
-
 def parse_args(argv):
 
     def get_extensions(extension_names):
         return [__import__('lizard' + name).LizardExtension() for name in extension_names]
 
     def get_whitelist():
-        try:
-            with open('whitelizard.txt', mode='r') as whitelizard:
-                return whitelizard.read()
-        except:
-            return ''
+        whitelist_filename = "whitelizard.txt"
+        if os.path.isfile(whitelist_filename):
+            return open(whitelist_filename, mode='r').read()
+        return ''
 
     options, args = createCommandLineParser().parse_args(args=argv)
     options.whitelist = get_whitelist()
